/*
 * Improved calculation of CRC-64 values for protein sequences
 * By David T. Jones (dtj@cs.ucl.ac.uk)  - September 28th 2002
 * Significantly modified by Simson Garfinkel - July 15, 2009
 * 
 * Modified from code at URL:
 * ftp://ftp.ebi.ac.uk/pub/software/swissprot/Swissknife/old/SPcrc.tar.gz
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>

/* If you want to try the old CRC-64 function, currently employed in
   SWISSPROT/TrEMBL then uncomment the next line */
/* #define OLDCRC */

#ifdef OLDCRC
  #define POLY64REV	0xd800000000000000ULL
  #define INITIALCRC	0x0000000000000000ULL
#else
  #define POLY64REV     0x95AC9329AC4BC9B5ULL
  #define INITIALCRC    0xFFFFFFFFFFFFFFFFULL
#endif



void crc64(u_char *buf, size_t buflen,u_char res[8])
{
    register int i, j;
    unsigned long long crc = INITIALCRC, part;
    static int init = 0;
    static unsigned long long CRCTable[256];
    
    if (!init)    {
	for (i = 0; i < 256; i++) {
	    part = i;
	    for (j = 0; j < 8; j++) {
		if (part & 1)
		    part = (part >> 1) ^ POLY64REV;
		else
		    part >>= 1;
	    }
	    CRCTable[i] = part;
	}
	init = 1;
    }
    
    for(i=0;i<buflen;i++){
	crc = CRCTable[(crc ^ buf[i]) & 0xff] ^ (crc >> 8);
    }

    (*(unsigned long *)(res+0)) = (crc >> 32) & 0xffffffff; /* high */
    (*(unsigned long *)(res+4)) = (crc & 0xffffffff); /* low */
    return;
}

