/*
 * Demonstrate the NSRL C++ class
 */

#include "config.h"
#include "bloom.h"
#include "hash_t.h"

#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <cstring>


int main(int argc,char **argv)
{
    int errors =0;
    NSRLBloom b;
    int M = 24;
    b.debug = 2;
    b.create(0,160,M,4,"A test bloom filter in memory");

    printf("Storing SHA1 values of even numbers into bloom filter...\n");
    for(int i=0;i<10;i+=2){
	char number[20];
	sprintf(number,"%d",i);
	const sha1_t s = sha1_generator::hash_buf((u_char *)number,strlen(number));
	printf("Storing i=%d\n",i);
	b.add(s.digest);
    }
    printf("Checking to make sure that odd numbers are not present and even numbers are present\n");
    for(int i=0;i<10;i++){
	char number[20];
	sprintf(number,"%d",i);
	const sha1_t s = sha1_generator::hash_buf((u_char *)number,strlen(number));
	printf("checking i=%d\n",i);
	bool present = b.query(s.digest);
	bool even     = (i%2)==0;

	if(present != even){
	    printf("error: i=%d  present=%d even=%d\n",i,present,even);
	    errors++;
	}
    }
    printf("done\n");
    if(errors>0) printf("errors=%d; M=%d; try a higher value\n",errors,M);
    return errors;
}
