/*
 * Demonstrate the encrypted bloom filters
 */

#include "config.h"

#include "bloom.h"

#include "hash_t.h"

#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <cstring>
#include <unistd.h>

int errors =0;
const char *fname = "cryptobloom.bf";

void save(const char *passphrase)
{
    printf("save: Storing SHA1 values of even numbers into bloom filter...\n");

    NSRLBloom b;
    if(b.create(fname,160,20,4,"A test of an encrypted bloom filter")){
	perror("Cannot create bloom filter");
	exit(1);
    }
    b.set_passphrase(passphrase);

    for(int i=0;i<1000;i+=2){
	char number[20];
	
	sprintf(number,"%d",i);

	const sha1_t s = sha1_generator::hash_buf((u_char *)number,strlen(number));
	b.add(s.digest);
    }
    printf("save done\n");
}

void retrieve(const char *passphrase,int should_find)
{
    printf("retrieve(%s,%d)\n",passphrase,should_find);

    NSRLBloom b;
    if(b.open(fname,O_RDONLY)){
	perror("Cannot open bloom filter");
	exit(1);
    }
    b.set_passphrase(passphrase);
    for(int i=0;i<1000;i++){
	char number[20];
	
	sprintf(number,"%d",i);

	const sha1_t s = sha1_generator::hash_buf((u_char *)number,strlen(number));
	bool present = b.query(s.digest);

	if(should_find){
	    bool even     = (i%2)==0;
	    if(present != even){
		printf("error: i=%d  present=%d even=%d\n",i,present,even);
		errors++;
	    }
	}
	else{
	    if(present){
		printf("error: i=%d present=%d but should_find=%d\n",i,present,should_find);
		errors++;
	    }
	}


    }
    printf("done\n");
}

int main(int argc,char **argv)
{
    unlink(fname);			// in case it's still there.
    save("this is a test");
    retrieve("this is a test",1);
    retrieve("",0);
    if(errors==0) unlink(fname);
    exit(errors);
}
