#ifndef SDBUF_H
#define SDBUF_H

#include <sys/types.h>
#include <openssl/md5.h>

class sdbuf {
public:
    uint8_t *buf;			// the actual buffer
    size_t  buflen;			// the length of the buffer
    bool    shouldfree;			// if true, buf is freed on deallocate
    size_t  block_size;	        // the size of the block
    int64_t block_number;
    
    histogram h;			// computed histogram

    /* Create the buffer; if copy is true, make a private copy. */
    sdbuf(size_t buflen){
	this->buflen = buflen;
	this->buf = (uint8_t *)calloc(buflen,1);
	this->shouldfree = true;
	this->block_size = buflen;	/* until we are told otherwise */
	this->block_number = -1;	/* don't know it. */
    }
    sdbuf(uint8_t *buf,size_t buflen,bool copy){
	this->buflen = buflen;
	this->shouldfree = copy;
	if(copy){
	    this->buf = (uint8_t *)malloc(buflen);
	    memcpy(this->buf,buf,buflen);
	}
	else{
	    this->buf = buf;
	}
    }
    void calc_block_histogram(){
	h.set(buf,block_size);
    }
    ~sdbuf(){
	if(shouldfree) free(buf);
    }
    /** returns true if all bytes in the block are null */
    bool null_block() const{
	for(size_t i=0;i<block_size;i++){
	    if(buf[i]) return false;
	}
	return true;
    }
    void calc_md5(uint8_t md5hash[16]) const{
	MD5(buf,block_size,md5hash);
    }
    void print_block(FILE *f) const{
	for(size_t i=0;i<block_size;i+=32){
	    for(size_t j=i;j<i+32 && j<block_size;j++){
		fprintf(f,"%02x ",buf[j]);
	    }
	    fprintf(f,"\n");
	}
    }
    uint8_t & operator[] (unsigned i) const{
	if(i>=buflen) throw "Invalid array access";
	return buf[i];
    }
    string substr(size_t offset,size_t len) const{
	return string((char *)buf+offset,len);
    }
};



#if 0
class contextbuf : public sdbuf {
 public:
    const uint8_t *idpoint;
    ssize_t buflen;
    void set(const uint8_t *buf,const uint8_t *idpoint,size_t buflen,int64_t block_number,
	     bool copy):sdbuf(buf,buflen,copy){
	this->idpoint = idpoint;
	this->block_number = block_number;
	this->h.set(this->idpoint,this->idbuflen());
    }
    ssize_t idbuflen() const{
	return buflen-(idpoint-buf);
    }
    int operator[] (ssize_t i) const{
	if(i<0 || i>= idbuflen()) return -1; /* invalid char */
	return idpoint[i];
    }
};
#endif

#endif
